﻿<#
.SYNOPSIS
	Gives the current user permission to access WMI classes on a remote
	computer running Windows.
	
.DESCRIPTION
	Set-WmiExplorerPermissions.ps1 gives the current user the permissions
	required to access WMI classes on a remote computer that is running
	Windows. These permissions are required to use SAPIEN Technologies, Inc.
	WMI Explorer on a remote computer. This script runs on Windows
	PowerShell 3.0 and later.
		
	This script has no parameters and takes no input. To use the script, run
	it locally on the target remote computer in an elevated session (Run as
	Administrator). Because it gives permission to the current user, be sure
	that the user account under which the script is run is the account that
	will be used to access the computer remotely.
		
	To get script status, use the Verbose parameter.
		
	For more information, see the blog post that describes this script:
	https://www.sapien.com/blog/2015/12/14/enable-wmi-explorer-windows-8-remote-connection-permissions/
		
	The script has four parts:
	1. Adds the current user to the Distributed COM Users group.
	2. Enables Remote Access to the WMI root namespace and sub-namespaces.
	3. Enables Remote Administration.
	4. Grants Windows Management Instrumentation (WMI) firewall access.
	
.EXAMPLE
	.\Set-WMIExplorerPermissions.ps1
	
.EXAMPLE
	.\Set-WMIExplorerPermissions.ps1 -Verbose
	
.NOTES
	This script was created with the help of the following scripts:
	Karl Mitschke:
	https://unlockpowershell.wordpress.com/2009/11/20/script-remote-dcom-wmi-access-for-a-domain-user/
		
	The Scripting Guy:
	https://blogs.technet.microsoft.com/heyscriptingguy/2014/10/03/adding-local-users-to-local-groups/
		
	08/02/2016: Added support for Windows 7 and 10.
		
	For questions, comments, or concerns, please visit us at the SAPIEN WMI
	Explorer Forum: https://www.sapien.com/forums/viewforum.php?f=24
		
	========================================================================
	Created with: 	SAPIEN Technologies, Inc., PowerShell Studio 2016
	Created on:   	10/20/2015 3:24 PM
	Created by:   	DevinL
	Organization: 	SAPIEN Technologies, Inc.
	Filename:     	Set-WMIExplorerPermissions.ps1
	========================================================================
#>
[CmdletBinding()]
param ()

#Requires -RunAsAdministrator
#Requires -Version 3.0

$ComputerName = $env:COMPUTERNAME
$Username = $env:USERNAME

<#
.SYNOPSIS
	Adds the current user to the Distributed COM Users group.

.DESCRIPTION
	This function adds the current user to the Distributed COM Users group 
	on the local computer. 

	If the user is already a member, it writes a verbose message and 
	continues. If the attempt to add the user fails, the function throws a 
	terminating error.

.EXAMPLE
	PS C:\> Add-UserToDCOM

.EXAMPLE
	PS C:\> Add-UserToDCOM -Verbose
#>
function Add-UserToDCOM {
    [CmdletBinding()]
    param ()
    
    Write-Verbose "Adding $Username to Distributed COM Users Group."
    
    $Group = [ADSI]"WinNT://$ComputerName/Distributed COM Users, Group"
    $User = "WinNT://$ComputerName/$Username, User"
    
    $GroupMembers = $Group.members() | ForEach-Object { [ADSI]$_ }
    if ($null -ne $GroupMembers.Path) {
        $MemberNames = Split-Path -Leaf $GroupMembers.Path
        
        if ($MemberNames -contains $Username) {
            Write-Verbose "$Username is already a member of the DCOM Users Group, skipping step."
        } else {
            try {
                $Group.Add($User)
                Write-Verbose "Added $Username to Distributed COM Users Group."
            } catch {
                # User doesn't have permission to modify the DCOM Users Group
                throw $_.Exception.Message
            }
        }
    }
}

<#
.SYNOPSIS
	Enables the Remote Enable permission for members of the Authenticated 
	Users Group on the root namespace and sub-namespaces.
	
.DESCRIPTION
	The Set-WMISecurity function grants members of the Authenticated Users 
	group Remote Enable permissions for the WMI root namespace and its 
	sub-namespaces.
	
	If it fails, it throws a terminating error.
	
.EXAMPLE
	PS C:\> Set-WMISecurity

.EXAMPLE
	PS C:\> Set-WMISecurity -Verbose

.NOTES
	This method was largely constructed by Karl Mitschke in his blog post:
	https://unlockpowershell.wordpress.com/2009/11/20/script-remote-dcom-wmi-access-for-a-domain-user/
		
	I made only minimal changes, such as adding CI to $SDDL to copy the 
	permissions to the sub-namespaces.
#>
function Set-WMISecurity {
    [CmdletBinding()]
    param ()
    
    Write-Verbose "Enabling Remote Enable to the root WMI namespace and sub-namespaces."
    
    # Assign the SID for Authenticated Users to the $SID var.
    $ID = New-Object System.Security.Principal.NTAccount('Authenticated Users')
    $SID = $ID.Translate([System.Security.Principal.SecurityIdentifier]).toString()
    
    # Permission string - Remote Enable across root namespace and subnamespaces
    $SDDL = "A;CI;CCWP;;;$SID"
    
    try {
        $Security = Get-WmiObject -ComputerName $ComputerName -Namespace Root -Class __SystemSecurity
        $Converter = New-Object System.Management.ManagementClass Win32_SecurityDescriptorHelper
        $BinarySD = @($null)
        
        $Result = $Security.PsBase.InvokeMethod('GetSD', $BinarySD)
        $OutSDDL = $Converter.BinarySDToSDDL($BinarySD[0])
        $NewSDDL = $OutSDDL.SDDL += '(' + $SDDL + ')'
        
        $WMIBinarySD = $Converter.SDDLToBinarySD($NewSDDL)
        $WMIConvertedPermissions = , $WMIBinarySD.BinarySD
        
        $Result = $Security.PsBase.InvokeMethod('SetSD', $WMIConvertedPermissions)
        Write-Verbose "Enabled Remote Enable."
    } catch {
        # User doesn't have permission to set WMI security
        throw $_.Exception.Message
    }
}

<#
.SYNOPSIS
	Enables the Remote Administration and WMI Rule Groups in the Windows 
	Firewall.

.DESCRIPTION
	Enables the Remote Administration and WMI Rule Groups in the Windows 
	Firewall. If the NetSecurity module is installed, it uses 
	Get-NetFirewallRule and Set-NetFirewallRule. Otherwise, it uses netsh 
	firewall. 

	If it fails, the script throws a terminating error.

.EXAMPLE
	PS C:\> Set-FirewallPermission

.EXAMPLE
    PS C:\> Set-FirewallPermission -Verbose
#>
function Set-FirewallPermission {
    [CmdletBinding()]
    param ()
    
    Write-Verbose "Enabling Windows Remote Management/Windows Management Instrumentation firewall rules."
    
    if (Get-Module -ListAvailable -Name NetSecurity) {
        if ((Get-NetFirewallRule -DisplayGroup "Windows Remote Management") -and (Get-NetFirewallRule -DisplayGroup "Windows Management Instrumentation (WMI)")) {
            try {
                Set-NetFirewallRule -DisplayGroup "Windows Management Instrumentation (WMI)" -Enabled True
                Set-NetFirewallRule -DisplayGroup "Windows Remote Management" -Enabled True
            } catch {
                # User doesn't have permission to set new Firewall rules
                throw $_.Exception.Message
            }
        } else {
            # WRM/WMI Rule Groups not present
            $null = netsh firewall set service RemoteAdmin enable
        }
    } else {
        # NetSecurity Module not present
        $null = netsh firewall set service RemoteAdmin enable
    }
    
    Write-Verbose "Set firewall rules."
}

Add-UserToDCOM

Set-WMISecurity

Set-FirewallPermission

Write-Verbose "Script execution completed."

# SIG # Begin signature block
# MIITFgYJKoZIhvcNAQcCoIITBzCCEwMCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUf39z2sf8h2Eezp/Zmc754vcm
# AX6ggg2lMIIEFDCCAvygAwIBAgILBAAAAAABL07hUtcwDQYJKoZIhvcNAQEFBQAw
# VzELMAkGA1UEBhMCQkUxGTAXBgNVBAoTEEdsb2JhbFNpZ24gbnYtc2ExEDAOBgNV
# BAsTB1Jvb3QgQ0ExGzAZBgNVBAMTEkdsb2JhbFNpZ24gUm9vdCBDQTAeFw0xMTA0
# MTMxMDAwMDBaFw0yODAxMjgxMjAwMDBaMFIxCzAJBgNVBAYTAkJFMRkwFwYDVQQK
# ExBHbG9iYWxTaWduIG52LXNhMSgwJgYDVQQDEx9HbG9iYWxTaWduIFRpbWVzdGFt
# cGluZyBDQSAtIEcyMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAlO9l
# +LVXn6BTDTQG6wkft0cYasvwW+T/J6U00feJGr+esc0SQW5m1IGghYtkWkYvmaCN
# d7HivFzdItdqZ9C76Mp03otPDbBS5ZBb60cO8eefnAuQZT4XljBFcm05oRc2yrmg
# jBtPCBn2gTGtYRakYua0QJ7D/PuV9vu1LpWBmODvxevYAll4d/eq41JrUJEpxfz3
# zZNl0mBhIvIG+zLdFlH6Dv2KMPAXCae78wSuq5DnbN96qfTvxGInX2+ZbTh0qhGL
# 2t/HFEzphbLswn1KJo/nVrqm4M+SU4B09APsaLJgvIQgAIMboe60dAXBKY5i0Eex
# +vBTzBj5Ljv5cH60JQIDAQABo4HlMIHiMA4GA1UdDwEB/wQEAwIBBjASBgNVHRMB
# Af8ECDAGAQH/AgEAMB0GA1UdDgQWBBRG2D7/3OO+/4Pm9IWbsN1q1hSpwTBHBgNV
# HSAEQDA+MDwGBFUdIAAwNDAyBggrBgEFBQcCARYmaHR0cHM6Ly93d3cuZ2xvYmFs
# c2lnbi5jb20vcmVwb3NpdG9yeS8wMwYDVR0fBCwwKjAooCagJIYiaHR0cDovL2Ny
# bC5nbG9iYWxzaWduLm5ldC9yb290LmNybDAfBgNVHSMEGDAWgBRge2YaRQ2XyolQ
# L30EzTSo//z9SzANBgkqhkiG9w0BAQUFAAOCAQEATl5WkB5GtNlJMfO7FzkoG8IW
# 3f1B3AkFBJtvsqKa1pkuQJkAVbXqP6UgdtOGNNQXzFU6x4Lu76i6vNgGnxVQ380W
# e1I6AtcZGv2v8Hhc4EvFGN86JB7arLipWAQCBzDbsBJe/jG+8ARI9PBw+DpeVoPP
# PfsNvPTF7ZedudTbpSeE4zibi6c1hkQgpDttpGoLoYP9KOva7yj2zIhd+wo7AKvg
# IeviLzVsD440RZfroveZMzV+y5qKu0VN5z+fwtmK+mWybsd+Zf/okuEsMaL3sCc2
# SI8mbzvuTXYfecPlf5Y1vC0OzAGwjn//UYCAp5LUs0RGZIyHTxZjBzFLY7Df8zCC
# BJ8wggOHoAMCAQICEhEh1pmnZJc+8fhCfukZzFNBFDANBgkqhkiG9w0BAQUFADBS
# MQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFsU2lnbiBudi1zYTEoMCYGA1UE
# AxMfR2xvYmFsU2lnbiBUaW1lc3RhbXBpbmcgQ0EgLSBHMjAeFw0xNjA1MjQwMDAw
# MDBaFw0yNzA2MjQwMDAwMDBaMGAxCzAJBgNVBAYTAlNHMR8wHQYDVQQKExZHTU8g
# R2xvYmFsU2lnbiBQdGUgTHRkMTAwLgYDVQQDEydHbG9iYWxTaWduIFRTQSBmb3Ig
# TVMgQXV0aGVudGljb2RlIC0gRzIwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEK
# AoIBAQCwF66i07YEMFYeWA+x7VWk1lTL2PZzOuxdXqsl/Tal+oTDYUDFRrVZUjtC
# oi5fE2IQqVvmc9aSJbF9I+MGs4c6DkPw1wCJU6IRMVIobl1AcjzyCXenSZKX1GyQ
# oHan/bjcs53yB2AsT1iYAGvTFVTg+t3/gCxfGKaY/9Sr7KFFWbIub2Jd4NkZrItX
# nKgmK9kXpRDSRwgacCwzi39ogCq1oV1r3Y0CAikDqnw3u7spTj1Tk7Om+o/SWJMV
# TLktq4CjoyX7r/cIZLB6RA9cENdfYTeqTmvT0lMlnYJz+iz5crCpGTkqUPqp0Dw6
# yuhb7/VfUfT5CtmXNd5qheYjBEKvAgMBAAGjggFfMIIBWzAOBgNVHQ8BAf8EBAMC
# B4AwTAYDVR0gBEUwQzBBBgkrBgEEAaAyAR4wNDAyBggrBgEFBQcCARYmaHR0cHM6
# Ly93d3cuZ2xvYmFsc2lnbi5jb20vcmVwb3NpdG9yeS8wCQYDVR0TBAIwADAWBgNV
# HSUBAf8EDDAKBggrBgEFBQcDCDBCBgNVHR8EOzA5MDegNaAzhjFodHRwOi8vY3Js
# Lmdsb2JhbHNpZ24uY29tL2dzL2dzdGltZXN0YW1waW5nZzIuY3JsMFQGCCsGAQUF
# BwEBBEgwRjBEBggrBgEFBQcwAoY4aHR0cDovL3NlY3VyZS5nbG9iYWxzaWduLmNv
# bS9jYWNlcnQvZ3N0aW1lc3RhbXBpbmdnMi5jcnQwHQYDVR0OBBYEFNSihEo4Whh/
# uk8wUL2d1XqH1gn3MB8GA1UdIwQYMBaAFEbYPv/c477/g+b0hZuw3WrWFKnBMA0G
# CSqGSIb3DQEBBQUAA4IBAQCPqRqRbQSmNyAOg5beI9Nrbh9u3WQ9aCEitfhHNmmO
# 4aVFxySiIrcpCcxUWq7GvM1jjrM9UEjltMyuzZKNniiLE0oRqr2j79OyNvy0oXK/
# bZdjeYxEvHAvfvO83YJTqxr26/ocl7y2N5ykHDC8q7wtRzbfkiAD6HHGWPZ1BZo0
# 8AtZWoJENKqA5C+E9kddlsm2ysqdt6a65FDT1De4uiAO0NOSKlvEWbuhbds8zkSd
# wTgqreONvc0JdxoQvmcKAjZkiLmzGybu555gxEaovGEzbM9OuZy5avCfN/61PU+a
# 003/3iCOTpem/Z8JvE3KGHbJsE2FUPKA0h0G9VgEB7EYMIIE5jCCA86gAwIBAgIQ
# D3G+iYSlUr2D5y/ELzPF6DANBgkqhkiG9w0BAQsFADB/MQswCQYDVQQGEwJVUzEd
# MBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xHzAdBgNVBAsTFlN5bWFudGVj
# IFRydXN0IE5ldHdvcmsxMDAuBgNVBAMTJ1N5bWFudGVjIENsYXNzIDMgU0hBMjU2
# IENvZGUgU2lnbmluZyBDQTAeFw0xNTA4MTIwMDAwMDBaFw0xODEwMTAyMzU5NTla
# MHkxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpDYWxpZm9ybmlhMQ0wCwYDVQQHEwRO
# YXBhMSIwIAYDVQQKFBlTQVBJRU4gVGVjaG5vbG9naWVzLCBJbmMuMSIwIAYDVQQD
# FBlTQVBJRU4gVGVjaG5vbG9naWVzLCBJbmMuMIIBIjANBgkqhkiG9w0BAQEFAAOC
# AQ8AMIIBCgKCAQEAwZqUHHSXh4kbcGg4h9gRD7D/ltVG5VI+/n5vfFAuKzcsftm9
# Kgh5kpjZHiuhG90in3qYQ2uYxyErTO1v7+399y+/7vjSh3MKf4VGhY5qieC08bb+
# 3z3zefGDp/9U3nNJj8YDpJ7lJWl4HDU5mnszlfUpKZQUYJ1Lj92EBvDsvukz8DJ2
# SrHwPQYZqL1qZ6v8uVraVVRhOQpFXCDq1QvzMt9xJGd/opRUvMasm0uvm/hS/kuJ
# 0TNtj9s8uKdUiM6KwlYyRZbdz/2B3l+LTjnKiXtkgnMakOLj1W9KjdDVrFDtGVMI
# taxI3yncU1qkSEftPd0+yZaefbnJS/UN2A6UwQIDAQABo4IBYjCCAV4wCQYDVR0T
# BAIwADAOBgNVHQ8BAf8EBAMCB4AwKwYDVR0fBCQwIjAgoB6gHIYaaHR0cDovL3N2
# LnN5bWNiLmNvbS9zdi5jcmwwZgYDVR0gBF8wXTBbBgtghkgBhvhFAQcXAzBMMCMG
# CCsGAQUFBwIBFhdodHRwczovL2Quc3ltY2IuY29tL2NwczAlBggrBgEFBQcCAjAZ
# DBdodHRwczovL2Quc3ltY2IuY29tL3JwYTATBgNVHSUEDDAKBggrBgEFBQcDAzBX
# BggrBgEFBQcBAQRLMEkwHwYIKwYBBQUHMAGGE2h0dHA6Ly9zdi5zeW1jZC5jb20w
# JgYIKwYBBQUHMAKGGmh0dHA6Ly9zdi5zeW1jYi5jb20vc3YuY3J0MB8GA1UdIwQY
# MBaAFJY7U/B5M5evfYPvLivMyreGHnJmMB0GA1UdDgQWBBQfoDZC6SEmEF6kHUon
# aYJz2Bv3nDANBgkqhkiG9w0BAQsFAAOCAQEAkKpfG9wjJ0gDATI4KTmQBgt0uiN9
# CWqVVO8P7j4RUANfVrwE0pQWPdtZOSDw3GURL98xWm6Dathkpv/FiGJL/6IRtfJ9
# 6zqfDL6rzI8pRpzWhPyxMMl7AImVfUpcEobJ+dnn2k1j9nF5UipkgapDPJUZqgBb
# UpNw69jKNwY0JCZPMt9CxsIrkxzsHJu072ZsXvYKmrD50GrKlvx5T4M5O4QEzU68
# 3qxfr5cYZzDQeKuH/v85tqg59d5N6tJnZxAjMCL6W24ToG6qQpsXcQRhYtLKR/OH
# q2teSjZhyyXzQEGETTYZA3SDkvjWI1+MlBk7jRyzXnqP2kZPIxLQlQF1hTGCBNsw
# ggTXAgEBMIGTMH8xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jw
# b3JhdGlvbjEfMB0GA1UECxMWU3ltYW50ZWMgVHJ1c3QgTmV0d29yazEwMC4GA1UE
# AxMnU3ltYW50ZWMgQ2xhc3MgMyBTSEEyNTYgQ29kZSBTaWduaW5nIENBAhAPcb6J
# hKVSvYPnL8QvM8XoMAkGBSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgACh
# AoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAM
# BgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBSd9dc1ZtE+6XHdS9Uu8P6mq7p9
# MTANBgkqhkiG9w0BAQEFAASCAQAmmr7jFYpiLAmEb9VTyvkxZ0Ouzd1gWtxhgghI
# 2oL3hWIltiAk6j3S2geHVjC73KZA90/Xfp9FKEhptiwDwvIOLDmzaUnB3XMwzXpX
# aLbPPs/mAG2J0B5rOkDU57RmllantxgmzO01uiBvnycxST2zSFdAjNmQkuknP0NV
# i83UQ7VxQ7pGvAMgTlziFGoi7Gz6DflAty4t9pZjOYdcFoGw54QEac/xR3OJvyrr
# nOoPTz/yqutt7C837X0pHAzW/IH91vHbufj01+Srs43Zt2irer9FgkPt7qhbJVvc
# XSbH5YfzevgfLW0+uY7qojmFc6acpv6n1fpxIZz8b2gEXMn/oYICojCCAp4GCSqG
# SIb3DQEJBjGCAo8wggKLAgEBMGgwUjELMAkGA1UEBhMCQkUxGTAXBgNVBAoTEEds
# b2JhbFNpZ24gbnYtc2ExKDAmBgNVBAMTH0dsb2JhbFNpZ24gVGltZXN0YW1waW5n
# IENBIC0gRzICEhEh1pmnZJc+8fhCfukZzFNBFDAJBgUrDgMCGgUAoIH9MBgGCSqG
# SIb3DQEJAzELBgkqhkiG9w0BBwEwHAYJKoZIhvcNAQkFMQ8XDTE2MDgzMTE5NDUw
# OVowIwYJKoZIhvcNAQkEMRYEFBxQYNjVWovBQnWs7mNT87GWshqNMIGdBgsqhkiG
# 9w0BCRACDDGBjTCBijCBhzCBhAQUY7gvq2H1g5CWlQULACScUCkz7HkwbDBWpFQw
# UjELMAkGA1UEBhMCQkUxGTAXBgNVBAoTEEdsb2JhbFNpZ24gbnYtc2ExKDAmBgNV
# BAMTH0dsb2JhbFNpZ24gVGltZXN0YW1waW5nIENBIC0gRzICEhEh1pmnZJc+8fhC
# fukZzFNBFDANBgkqhkiG9w0BAQEFAASCAQCmRD1hXHSzMskUMj8NzNBXFBHHOeEY
# FW/X2Yd+n92ZcyBpKUslnX6+jaIvuRbKhTylfT4yNcKG+BVmQgU90Wp7KRczlfru
# kQKLdyriWBMKz5LC9+xGCwjnf3+Pl6JNayuJRTZxbSMHJqa+UbS5dtdtyM4SrUxV
# NnTj/XqMEWyLnBZyOZv6Dg6SZPmF5j8XNTuqeBsBJRGra4DYBMOrZaFqi9b+yL9m
# nvfnSwY8LGhQWYxN4tlqBoiOOipNtXlvL7b2JtKLRG9n8YbUiNZNXVqgeAefMqLQ
# HBvKl3Qt8OmY0mclNvff6nGTglS1WOuE4HhlqaQ7JK5JVy+GWutyxcGp
# SIG # End signature block
